/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.concurrent.Executor;

import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.JAXBHelper;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.model.ActionsBuilder;
import com.redhat.rhevm.api.model.StorageDomain;
import com.redhat.rhevm.api.model.StorageDomainStatus;
import com.redhat.rhevm.api.model.Template;
import com.redhat.rhevm.api.model.Templates;
import com.redhat.rhevm.api.model.VM;
import com.redhat.rhevm.api.model.VMs;
import com.redhat.rhevm.api.resource.AssignedPermissionsResource;
import com.redhat.rhevm.api.resource.StorageDomainContentsResource;
import com.redhat.rhevm.api.resource.StorageDomainResource;
import com.redhat.rhevm.api.resource.FilesResource;
import com.redhat.rhevm.api.common.util.StatusUtils;

public class MockStorageDomainResource extends AbstractMockResource<StorageDomain> implements StorageDomainResource {

    /**
     * Package-protected ctor, never needs to be instantiated by JAX-RS framework.
     *
     * @param storageDomain  encapsulated StorageDomain
     * @param executor       executor used for asynchronous actions
     */
    MockStorageDomainResource(String id, Executor executor,  UriInfoProvider uriProvider) {
        super(id, executor, uriProvider);
        getModel().setStatus(StatusUtils.create(StorageDomainStatus.UNATTACHED));
    }

    // FIXME: this needs to be atomic
    public void updateModel(StorageDomain storageDomain) {
        // update writable fields only
        if (storageDomain.isSetName()) {
            getModel().setName(storageDomain.getName());
        }
        if (storageDomain.isSetDescription()) {
            getModel().setDescription(storageDomain.getDescription());
        }
    }

    public StorageDomain addLinks() {
        StorageDomain storageDomain = JAXBHelper.clone(OBJECT_FACTORY.createStorageDomain(getModel()));

        storageDomain = LinkHelper.addLinks(getUriInfo(), storageDomain);

        UriBuilder uriBuilder = LinkHelper.getUriBuilder(getUriInfo(), storageDomain);

        ActionsBuilder actionsBuilder = new ActionsBuilder(uriBuilder, StorageDomainResource.class);
        storageDomain.setActions(actionsBuilder.build());

        return storageDomain;
    }

    @Override
    public StorageDomain get() {
        return addLinks();
    }

    @Override
    public StorageDomain update(StorageDomain storageDomain) {
        validateUpdate(storageDomain);
        updateModel(storageDomain);
        return addLinks();
    }

    @Override
    public AssignedPermissionsResource getPermissionsResource() {
        return null;
    }

    public StorageDomainContentsResource<VMs, VM> getStorageDomainVmsResource() {
        return null;
    }

    public StorageDomainContentsResource<Templates, Template> getStorageDomainTemplatesResource() {
        return null;
    }

    @Override
    public FilesResource getFilesResource() {
       return null;
    }
}
