/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.mock.util.SimpleQueryEvaluator;
import com.redhat.rhevm.api.model.Storage;
import com.redhat.rhevm.api.model.StorageDomain;
import com.redhat.rhevm.api.model.StorageDomains;
import com.redhat.rhevm.api.model.StorageDomainType;
import com.redhat.rhevm.api.model.StorageType;
import com.redhat.rhevm.api.resource.StorageDomainResource;
import com.redhat.rhevm.api.resource.StorageDomainsResource;

import static com.redhat.rhevm.api.mock.resource.AbstractMockResource.allocateId;


public class MockStorageDomainsResource extends AbstractMockQueryableResource<StorageDomain> implements StorageDomainsResource {

    private static Map<String, MockStorageDomainResource> storageDomains =
        Collections.synchronizedMap(new HashMap<String, MockStorageDomainResource>());

    public MockStorageDomainsResource() {
        super(new SimpleQueryEvaluator<StorageDomain>());
    }

    private void addStorageDomain(StorageDomainType domainType, String name, StorageType storageType, String address, String path) {
        MockStorageDomainResource resource = new MockStorageDomainResource(allocateId(StorageDomain.class), getExecutor(), this);

        resource.getModel().setName(name);
        resource.getModel().setType(domainType.value());

        Storage storage = new Storage();
        storage.setType(storageType.value());
        storage.setAddress(address);
        storage.setPath(path);
        resource.getModel().setStorage(storage);

        storageDomains.put(resource.getModel().getId(), resource);
    }

    public void populate() {
        synchronized (storageDomains) {
            if (storageDomains.size() == 0) {
                addStorageDomain(StorageDomainType.DATA, "images_0", StorageType.NFS, "172.31.0.6", "/exports/RHEV/images/0");
                addStorageDomain(StorageDomainType.ISO, "isos_0", StorageType.NFS, "172.31.0.6", "/exports/RHEV/iso/0");
            }
        }
    }

    @Override
    public StorageDomains list() {
        StorageDomains ret = new StorageDomains();

        for (MockStorageDomainResource storageDomain : storageDomains.values()) {
            if (filter(storageDomain.getModel(), getUriInfo(), StorageDomain.class)) {
                ret.getStorageDomains().add(storageDomain.addLinks());
            }
        }

        return ret;
    }

    @Override
    public Response add(StorageDomain storageDomain) {
        MockStorageDomainResource resource = new MockStorageDomainResource(allocateId(StorageDomain.class), getExecutor(), this);

        resource.updateModel(storageDomain);

        String id = resource.getId();
        storageDomains.put(id, resource);

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(id);

        storageDomain = resource.addLinks();

        return Response.created(uriBuilder.build()).entity(storageDomain).build();
    }

    @Override
    public void remove(String id, StorageDomain storageDomain) {
        storageDomains.remove(id);
    }

    @Override
    public StorageDomainResource getStorageDomainSubResource(String id) {
        return storageDomains.get(id);
    }
}
